/*
 * hv7131gp_hal.h
 *
 *  Created on: 6-feb-2009
 *  Author: Claudio Salvadori
 *  Hardware abstraction level of CMOS Image Sensor HV7131GP using FLEX
 *  (with dsPIC33FJ256).
 */
#include "hv7131gp.h"

#ifndef HV7131GP_HAL_H_
#define HV7131GP_HAL_H_

#include "i2c.h"
#include "ee.h"
#include "cpu/pic30/inc/ee_irqstub.h"

/******************************************************************************/
/*                         Global Variables                                   */
/******************************************************************************/
/*#ifndef _HV7131GP_HAL_C_

extern volatile uint8_t *frame_buffer;

extern volatile void (* capture_complete_func) (hv7131gp_status_t);
extern volatile void (* timeout_func) (void);

#ifdef Debug
extern volatile unsigned int d1, d2;
#endif

#endif*/

/******************************************************************************/
/*                           Hardware Abstraction Layer                       */
/******************************************************************************/
/**
* @name Hardware Abstraction Layer
*
* \todo Write more
*
* @{ */
#define HV7131GP_PIN_ENABLE_INIT() 	TRISDbits.TRISD2=0
#define HV7131GP_PIN_ENABLE_HIGH() 	LATDbits.LATD2=1
#define HV7131GP_PIN_ENABLE_LOW() 	LATDbits.LATD2=0

#define HV7131GP_PIN_RESETB_INIT() 	TRISDbits.TRISD1=0
#define HV7131GP_PIN_RESETB_HIGH() 	LATDbits.LATD1=1
#define HV7131GP_PIN_RESETB_LOW() 	LATDbits.LATD1=0

#ifdef Debug
#define HV7131GP_PIN_DEBUG_INIT()  	TRISGbits.TRISG1 = 0
#endif

/**
* @name Vertical Sync
*
* INT3. \todo write more
*
* @{ */
#define HV7131GP_PIN_VSYNC_INIT_POSITIVE()					\
do { 									\
	INTCON2bits.INT3EP = 0;	/* Extern Interrupt on positive edge */	\
	IPC13bits.INT3IP  = 6;	/* Set INT3 interrupt priority */	\
} while (0)								\

#define HV7131GP_PIN_VSYNC_INIT_NEGATIVE()					\
do { 									\
	INTCON2bits.INT3EP = 1;	/* Extern Interrupt on negative edge */	\
	IPC13bits.INT3IP  = 6;	/* Set INT3 interrupt priority */	\
} while (0)								\

#define HV7131GP_PIN_VSYNC_START()					\
do { 									\
	IEC3bits.INT3IE = 1;	/* Enable INT3 interrupt */		\
	IFS3bits.INT3IF = 0;	/* Reset INT3 interrupt flag */	 	\
} while (0)								\

#define HV7131GP_PIN_VSYNC_STOP()					\
do { 									\
	IEC3bits.INT3IE = 0;	/* Disable INT3 interrupt */		\
	IFS3bits.INT3IF = 0;	/* Reset INT3 interrupt flag */	 	\
} while (0)								\

#define HV7131GP_PIN_VSYNC_IF_RESET()					\
do { 									\
	IFS3bits.INT3IF = 0;	/* Reset INT3 interrupt flag */	 	\
} while (0)								\

#define HV7131GP_PIN_VSYNC_IF_VALUE	IFS3bits.INT3IF
/**  @} */

/**
* @name Horizontal Sync
*
* CN15 (RD6). \todo write more
*
* @{ */
#define HV7131GP_PIN_HSYNC_INIT()					\
do { 									\
	TRISDbits.TRISD6 = 1;	/* Set CN15 (RD6) as digital input */	\
	IPC4bits.CNIP = 6;	/* Set CN interrupt priority */		\
} while (0)								\

#define HV7131GP_PIN_HSYNC_START()					\
do { 									\
	CNEN1bits.CN15IE = 1;	/* Enable CN15(RD6) change interrupt */	\
	IEC1bits.CNIE = 1;	/* Enable CN interrupt */		\
	IFS1bits.CNIF = 0;	/* Reset CN interrupt flag */ 		\
} while (0)								\

#define HV7131GP_PIN_HSYNC_STOP()					\
do { 									\
	IEC1bits.CNIE = 0;	/* Disable CN interrupt */		\
	CNEN1bits.CN15IE = 0;	/* Disable CN15(RD6) change interrupt */\
	IFS1bits.CNIF = 0;	/* Reset CN interrupt flag */	 	\
} while (0)									\

/**  @} */

/**
* @name Video (Pixels) Clock
*
* INT4. \todo write more
*
* @{ */
#define HV7131GP_PIN_VCLK_INIT()					\
do { 									\
	INTCON2bits.INT4EP = 0;	/* Extern Interrupt on positive edge */	\
	IPC13bits.INT4IP  = 6;	/* Set INT4 interrupt priority */	\
} while (0)								\

#define HV7131GP_PIN_VCLK_START()					\
do { 									\
	IEC3bits.INT4IE = 1;	/* Enable INT4 interrupt */		\
	IFS3bits.INT4IF = 0;	/* Reset INT4 interrupt flag */	 	\
} while (0)								\

#define HV7131GP_PIN_VCLK_STOP()					\
do { 									\
	IEC3bits.INT4IE = 0;	/* Disable INT4 interrupt */		\
	IFS3bits.INT4IF = 0;	/* Reset INT4 interrupt flag */	 	\
} while (0)								\

/**  @} */


/**
* @name Master Clock
*
* OC1 with Timer3. \todo write more
*
* @{ */

/* f[?], fcy[?], t[?] */
#define HV7131GP_PIN_MCLK_INIT()	/*TODO?*/
#define HV7131GP_PIN_MCLK_START(f)	oc1_start((f))

/**
* @brief Stop OC1 Interface
*/
#define HV7131GP_PIN_MCLK_STOP()					\
do{ 									\
	T3CON = 0;		/* Stops Timer3 and reset control reg */\
	TMR3 = 0;		/* Clear the timer register */		\
	LATDbits.LATD0 = 0;	/* Set line to low level */		\
} while (0)								\


/**
* @name Pixel pins
* @{ */
#define HV7131GP_PIN_Y_INIT()	TRISE = 0x00FF
#define HV7131GP_PIN_Y_VALUE	PORTE
/**  @} */

/**
* @name Timeout
*
* Timer5. \todo write more
*
* @{ */
#define HV7131GP_TIMEOUT_INIT(fcy)	timer5_timeout_init((fcy))
#define HV7131GP_TIMEOUT_SET(t)		timer5_timeout_set((t))

/**
* @brief Stop timer5 for timeout
*/
#define HV7131GP_TIMEOUT_RESET()					\
do{ 									\
	T5CONbits.TON = 0;	/* Stop Timer5 */			\
	IFS1bits.T5IF = 0;	/* Clear Timer5 interrupt status flag */\
	IEC1bits.T5IE = 0;	/* Disable Timer5 interrupts */		\
} while (0)								\

/**
* @name Software interrupt: end of frame acquisition
*
* INT0. \todo write more
*
* @{ */
#define HV7131GP_PIN_EOF_INIT()					\
do { 									\
	INTCON2bits.INT0EP = 1;	/* Extern Interrupt on negative edge */	\
	IPC0bits.INT0IP  = 6;	/* Set INT0 interrupt priority */	\
	IEC0bits.INT0IE = 1;	/* Enable INT0 interrupt */		\
	IFS0bits.INT0IF = 0;	/* Reset INT0 interrupt flag */	 	\
} while (0)								\

#define HV7131GP_PIN_EOF_STOP()					\
do { 									\
	IEC0bits.INT0IE = 0;	/* Disable INT0 interrupt */		\
	IFS0bits.INT0IF = 0;	/* Reset INT0 interrupt flag */	 	\
} while (0)								\

/**  @} */

#endif /* HV7131GP_HAL_H_ */
