#include "ee.h"
#include "cpu/pic30/inc/ee_irqstub.h"

#include "hv7131gp.h"
#include "utils.h"

/* Primary (XT, HS, EC) Oscillator without PLL */
_FOSCSEL(FNOSC_PRIPLL);
/* OSC2 Pin Function: OSC2 is Clock Output
 * Primary Oscillator Mode: XT Crystanl
*/
_FOSC(OSCIOFNC_ON & POSCMD_XT);
/* Watchdog Timer Enabled/disabled by user software */
_FWDT(FWDTEN_OFF);
/* Disable Code Protection */
_FGS(GCP_OFF);

/******************************************************************************/
/*                         Forward Declarations                               */
/******************************************************************************/
typedef enum {
	TRUE 		= 0x00,	/**< True */
	FALSE		= 0x01, /**< False */
} Boolean;

/******************************************************************************/
/*                         Global Variables                                   */
/******************************************************************************/
volatile unsigned char image[19600] __attribute__((far));
volatile unsigned char pippo = 0;
volatile unsigned char minnie = 0;

volatile UART_Status Stat = Status0;
volatile unsigned char buff[20];
Boolean sp = TRUE; //send picture or not
hv7131gp_Q_Value_t Quality  = HV7131GP_160x120_MEDIUM;
hv7131gp_T_D_Value_t Divisor = HV7131GP_T_1;
char sleep = 0;

uint16_t HBLANK  = 0xD0;
uint16_t VBLANK  = 0x08;

/******************************************************************************/
/*                         Forward Declarations                               */
/******************************************************************************/
void timer_set(void);
void process_frame(/*hv7131gp_status_t status*/);
Boolean Check_msg(void);
char get_numeric(char v1, char v2);

/******************************************************************************/
/*                            INLINE Functions                                */
/******************************************************************************/
/* ... */

/******************************************************************************/
/*                                  TASKs                                     */
/******************************************************************************/

/* ----------------------------------------------------------------------------
|  Task for testing the function sleep and activate                            |
|                                                                              |
|  Activated by Alarm1, set periodically camera in sleep/active status         |
-------------------------------------------------------------------------------*/

TASK(Command)
{
	switch(buff[1])
	{
	case 0x52: //R
		switch(buff[4])
		{
		case 0x30:
			if (hv7131gp_configure_resolution(HV7131GP_160x120_FAST)
			    != HV7131GP_SUCCESS)
				for(;;);
			break;
		case 0x31:
			if (hv7131gp_configure_resolution(HV7131GP_160x120_MEDIUM)
			    != HV7131GP_SUCCESS)
				for(;;);
			break;

		case 0x32:
		default:
			if (hv7131gp_configure_resolution(HV7131GP_160x120_SLOW)
			    != HV7131GP_SUCCESS)
				for(;;);
			break;
		}
		break;
	case 0x54: //T
		switch(buff[4])
		{
		case 0x30:
			if (hv7131gp_configure_time_divisor(HV7131GP_T_1)
			    != HV7131GP_SUCCESS)
				for(;;);
			break;
		case 0x31:
			if (hv7131gp_configure_time_divisor(HV7131GP_T_2)
			    != HV7131GP_SUCCESS)
				for(;;);
			break;

		case 0x32:
			if (hv7131gp_configure_time_divisor(HV7131GP_T_3)
			    != HV7131GP_SUCCESS)
				for(;;);
			break;
		case 0x33:
			if (hv7131gp_configure_time_divisor(HV7131GP_T_4)
			    != HV7131GP_SUCCESS)
				for(;;);
			break;

		case 0x34:
			if (hv7131gp_configure_time_divisor(HV7131GP_T_5)
			    != HV7131GP_SUCCESS)
				for(;;);
			break;
		case 0x35:
			if (hv7131gp_configure_time_divisor(HV7131GP_T_6)
			    != HV7131GP_SUCCESS)
				for(;;);
			break;
		case 0x36:
			if (hv7131gp_configure_time_divisor(HV7131GP_T_7)
			    != HV7131GP_SUCCESS)
				for(;;);
			break;
		case 0x37:
		default:
			if (hv7131gp_configure_time_divisor(HV7131GP_T_8)
			    != HV7131GP_SUCCESS)
				for(;;);
			break;
		}
		break;

	case 0x53: //S
		if (hv7131gp_set_sleep_status() != HV7131GP_SUCCESS)
			for(;;);
		break;

	case 0x41: //A
		if (hv7131gp_set_active_status() != HV7131GP_SUCCESS)
			for(;;);
		break;

	case 0x48: //H
		if (hv7131gp_hblank_set((int)((buff[3] << 8) + buff[4]))
				!= HV7131GP_SUCCESS)
			for(;;);
		if (hv7131gp_vblank_set((int)((buff[6] << 8) + buff[7]))
				!= HV7131GP_SUCCESS)
			for(;;);
		break;

	case 0x44: //D
		if (hv7131gp_window_set((int)((buff[3] << 8) + buff[4]),
					(int)((buff[6] << 8) + buff[7]),
					(int)((buff[9] << 8) + buff[10]),
					(int)((buff[12] << 8) + buff[13]))
			!= HV7131GP_SUCCESS)
			for(;;);
		break;

	case 0x31: //1
		pippo = get_numeric(buff[3], buff[4]);
		minnie = get_numeric(buff[6], buff[7]);
		if (hv7131gp_reg_write(get_numeric(buff[3], buff[4]),
				       get_numeric(buff[6], buff[7]))
			!= HV7131GP_SUCCESS)
			for(;;);
		break;

	default:
		memset(buff, 0, 20);
		return FALSE;
		break;
	}

	/* Enable RX Interrupts */
	IEC0bits.U1RXIE = 1;

}
/* ----------------------------------------------------------------------------
|  Receive bytes from Pc (Serial communication)                                |
|                                                                              |
|  Reception of the init and settings message and answer (ack, nack)           |
-------------------------------------------------------------------------------*/

TASK(Receive)
{
	const unsigned char ack = 0x31;
	const unsigned char nak = 0x30;
	unsigned int i;

	switch (Stat)
	{
	case Status0:
		pc_serial_read(&buff[0]); //init msg
		if (buff[0] == 0x31)
		{
			pc_serial_write(&ack, 1);
			Stat = Status1;
		}
		else
			pc_serial_write(&nak, 1);
		break;
	case Status1:
		IEC0bits.U1RXIE = 0;

		//Fill the command buffer
		for(i=0; i<7; i++)
		//for(i=0; i<20; i++)
		{
			while (!IFS0bits.U1RXIF);

			pc_serial_read (&buff[i]);

			IFS0bits.U1RXIF=0;
		}

		//Check the command form
		if (Check_msg() == TRUE)
		{
			pc_serial_write(&ack, 1);
			Stat = Status2;
		}
		else
			pc_serial_write(&nak, 1);

		break;
	}
	memset(buff, 0, 20);
}

/******************************************************************************/
/*                                   MAIN                                     */
/******************************************************************************/
int main(void)
{
	/* Clock setup for 40 MHz */
	CLKDIVbits.DOZEN   = 0;
	CLKDIVbits.PLLPRE  = 0;
	CLKDIVbits.PLLPOST = 0;
	PLLFBDbits.PLLDIV  = 78;
	while (OSCCONbits.LOCK!=1) ;

	unsigned char m_2test[2];


	/* Init system timer: Timer 1*/
	timer_set();

	/* Init components */
	EE_leds_init();

	/* Initilizes the PC serial link (RS232) */
	pc_serial_init(115200);

	EE_led_on();

	m_2test[0]=49;	m_2test[1]=50;
	pc_serial_write(m_2test, 2);
	/* Initializes the camera device */
	if (hv7131gp_init() != HV7131GP_SUCCESS)
		for (;;) ;

	Stat = Status0;

	m_2test[0]='A';	m_2test[1]='B';
	pc_serial_write(m_2test, 2);

	/*Wait for serial command (0 = start)*/
	/*while (Stat != Status1)
	{
		if (IFS0bits.U1RXIF)
		{
			ActivateTask(Receive); //Command reception

			IFS0bits.U1RXIF = 0;
		}
	}*/

	m_2test[0]='C';	m_2test[1]='D';
	pc_serial_write(m_2test, 2);
	// Wait for capture state
	//while (!IFS0bits.U1RXIF);

	//ActivateTask(Receive); //Command reception

	m_2test[0]='E';	m_2test[1]='F';
	pc_serial_write(m_2test, 2);

	//while (Stat != Status2);

	/* Configure the camera device */
	if (hv7131gp_configure(Quality) != HV7131GP_SUCCESS)
		for (;;) ;

	m_2test[0]='G';	m_2test[1]='H';
	pc_serial_write(m_2test, 2);

	EE_led_off();

	/* Enable RX Interrupts */
	IEC0bits.U1RXIE = 1;

	/* Initializes the frame capture from camera */
	if (hv7131gp_capture(image, process_frame) != HV7131GP_SUCCESS)
		for (;;) ;

	m_2test[0]='I';	m_2test[1]='J';
	pc_serial_write(m_2test, 2);
	/* Forever loop: background activities (if any) should go here */
	for (;;) ;

	return 0;
}

/******************************************************************************/
/*                          Function Definitions                              */
/******************************************************************************/
/* ----------------------------------------------------------------------------
|  Setup system timers:                                                        |
|                                                                              |
|  Timer1: used for system ticks (CounterTick)                                 |
 ---------------------------------------------------------------------------- */
void
timer_set(void)
{
	/* TIMER 1 */

	T1CON = 0;		/* Stops the Timer1 and reset control reg */
	TMR1 = 0;		/* Clear contents of the timer register	*/
	PR1 = 0x9C40;		/* Load the Period register -> 1ms @ 40MHz */
	IPC0bits.T1IP = 5;	/* Set Timer1 priority to 5 */
	IFS0bits.T1IF = 0;	/* Clear the Timer1 interrupt status flag */
	IEC0bits.T1IE = 1;	/* Enable Timer1 interrupts */
	T1CONbits.TON = 1;	/* Start Timer1 without prescaler */
}

/* ----------------------------------------------------------------------------
|  Process Frame Callback:                                                     |
|                                                                              |
|  Send the read image via PC serial link and start a new frame capture        |
 ---------------------------------------------------------------------------- */
void
process_frame(void/*hv7131gp_status_t status*/)
{
	const unsigned char header[3] = {0xAA, 0xBB, 0xCC};
	unsigned char m_2byte[2];
	unsigned char w, h, y, l;
	unsigned int s;
	volatile unsigned int dh = 0;
	volatile unsigned int dl = 0;

	//Sent values
	y = hv7131gp_get_Y_average();
	w = hv7131gp_get_width();
	h = hv7131gp_get_height();
	s = hv7131gp_get_size();
	dh = hv7131gp_get_fdh();
	dl = hv7131gp_get_fdl();
	l =  hv7131gp_get_fl();

	EE_led_on();
	pc_serial_write(header, 3);

	//Send width
	m_2byte[0] = ((unsigned int) w >> 8) & 0xFF;
	m_2byte[1] = ((unsigned int) w     ) & 0xFF;
	pc_serial_write(m_2byte, 2);

	//Send Height
	m_2byte[0] = ((unsigned int) h >> 8) & 0xFF;
	m_2byte[1] = ((unsigned int) h     ) & 0xFF;
	pc_serial_write(m_2byte, 2);

	//Frame duration (part 1)
	m_2byte[0] = ((unsigned int) dh >> 8) & 0xFF;
	m_2byte[1] = ((unsigned int) dh     ) & 0xFF;
	pc_serial_write(m_2byte, 2);

	//Frame duration (part 2)
	m_2byte[0] = ((unsigned int) dl >> 8) & 0xFF;
	m_2byte[1] = ((unsigned int) dl     ) & 0xFF;
	pc_serial_write(m_2byte, 2);

	//have I lose a frame??
	m_2byte[0] = ((unsigned int) l	 >> 8) & 0xFF;
	m_2byte[1] = ((unsigned int) l	     ) & 0xFF;
	pc_serial_write(m_2byte, 2);

	//Y mean value
	m_2byte[0] = ((unsigned int) y >> 8) & 0xFF;
	m_2byte[1] = ((unsigned int) y     ) & 0xFF;
	pc_serial_write(m_2byte, 2);

	if (sp == TRUE)
		pc_serial_write(image, s);


	EE_led_off();

	if (hv7131gp_capture(image, process_frame) != HV7131GP_SUCCESS)
		for (;;) ;
}

/* ----------------------------------------------------------------------------
|  Check received message                                                      |
|                                                                              |
|  Check if the message received is formally correct and update the settings   |
 ---------------------------------------------------------------------------- */
Boolean
Check_msg(void)
{
	//Check the "*"
	if (buff[0] != 0x2A)
		return FALSE;

	if (buff[3] != 0x2A)
		return FALSE;

	if (buff[6] != 0x2A)
		return FALSE;

	switch (buff[2])
	{
		case 0x30:
		Quality = HV7131GP_160x120_FAST;
		break;

		case 0x31:
		Quality = HV7131GP_160x120_MEDIUM;
		break;

		case 0x32:
		Quality = HV7131GP_160x120_SLOW;
		break;

		default:
		return FALSE;
		break;
	}

	//Send image??

	if (buff[4] != 0x00)
		return FALSE;

	switch (buff[5])
	{
		case 0x30:
		sp = FALSE;
		break;

		case 0x31:
		sp = TRUE;
		break;

		default:
		return TRUE;
		break;
	}

	return TRUE;

}
/* ----------------------------------------------------------------------------
|  Function that transform ascii in char                                       |
|                                                                              |
|  Return a char value from 2 ascii values				       |
 ---------------------------------------------------------------------------- */
char get_numeric(char v1, char v2)
{
	unsigned char h1 = 0;
	unsigned char h2 = 0;
	if((v1 >= 0x30) && (v1 <= 0x39))
		h1 = v1 - 0x30;
	else{
		if((v1 >= 0x41) && (v1 <= 0x46))
			h1 = v1 - 0x37;
	}

	if((v2 >= 0x30) && (v2 <= 0x39))
		h2 = v2 - 0x30;
	else{
		if((v2 >= 0x41) && (v2 <= 0x46))
			h2 = v2 - 0x37;
	}

	return ((h1 << 4) + h2);
}
/******************************************************************************/
/*                                     ISRs                                   */
/******************************************************************************/
/* ----------------------------------------------------------------------------
|  ISR2 Timer1:                                                                |
|                                                                              |
|  Calls CounterTick for task_counter                                          |
 ---------------------------------------------------------------------------- */
ISR2(_T1Interrupt)
{
	IFS0bits.T1IF = 0;
	CounterTick(task_counter);
}

/* ----------------------------------------------------------------------------
|  ISR2 U1RX:                                                                  |
|                                                                              |
|  Settings (during capture) are arriving                                      |
 ---------------------------------------------------------------------------- */
ISR2(_U1RXInterrupt)
{
	int i;
	memset(buff, 0, 20);

	//RECEIVE: settings command and set a
	IEC0bits.U1RXIE = 0;
	pc_serial_read (&buff[0]);
	IFS0bits.U1RXIF=0;
	//Fill the command buffer
	for(i=1; i<15; i++){
		while (!IFS0bits.U1RXIF);
		pc_serial_read (&buff[i]);
		IFS0bits.U1RXIF=0;
	}
	ActivateTask(Command); //Command reception during capture
	IFS0bits.U1RXIF=0;
}

/******************************************************************************/
/*                                    TRAPs                                   */
/******************************************************************************/
/*
void _ISRFAST _OscillatorFail(void) { for(;;); }

void _ISRFAST _AddressError(void) { for(;;); }

void _ISRFAST _StackError(void) { for(;;); }

void _ISRFAST _MathError(void) { for(;;); }

void _ISRFAST _DMACError(void) { for(;;); }
*/
