/*
Copyright (C) 2007 - Nastasi Christian
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _myUtils_h_
#define _myUtils_h_

#include <stdio.h>
#include <jpeglib.h>

typedef struct
{
	struct jpeg_source_mgr pub;   
	unsigned char* buffer;
	unsigned int   bufsize;
} myJPEG_source_mgr;

typedef myJPEG_source_mgr* myJPEG_src_ptr;

/** 
* @file myUtils.h
* @brief Conversion functions
* @author Nastasi Christian
* @date 2007-12-17
*/


/** 
* @brief JPEG Decompression
* 
* @param srcBuffer Source JPEG image
* @param size Source JPEG image size
* @param dstBuffer Destination buffer capable to store the decompressed image
* @param width The decompressed image width
* @param height The decompressed image height
* @param bpp The decompressed image number of byte per pixel
* 
* @return 
*/
bool my_JPEG_decompress(unsigned char* srcBuffer, int size, unsigned char* dstBuffer, int& width, int& height, int& bpp);

/** 
* @brief Convert from RGB16 to RGB24
* 
* @param src Source image
* @param dest Destination buffer capable to store the result image
* @param width Image width
* @param height Image height
*/
void my_rgb16_to_rgb24(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height);
/** 
* @brief Convert from RGB12 to RGB24
* 
* @param src Source image
* @param dest Destination buffer capable to store the result image
* @param width Image width
* @param height Image height
*/
void my_rgb12_to_rgb24(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height);
/** 
* @brief Convert from Gray4 to Gray8
* 
* @param src Source image
* @param dest Destination buffer capable to store the result image
* @param width Image width
* @param height Image height
*/
void my_gray4_to_gray8(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height);
/** 
* @brief Convert from Gray2 to Gray8
* 
* @param src Source image
* @param dest Destination buffer capable to store the result image
* @param width Image width
* @param height Image height
*/
void my_gray2_to_gray8(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height);

/** 
* @brief Convert from Gray8 to RGB24 
* 
* @param src Source image
* @param dest Destination buffer capable to store the result image
* @param width Image width
* @param height Image height
*/
void my_gray8_to_rgb24(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height);

/** 
* @brief Convert from Black\White Binary to Gray8
* 
* @param src Source image
* @param dest Destination buffer capable to store the result image
* @param width Image width
* @param height Image height
*/
void my_bwBin_to_gray8(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height);


typedef enum {
	MY_RED,
	MY_GREEN,
	MY_BLUE,
}My_draw_color;

/** 
* @brief Mark a point with a red cross
* 
* @param im RGB24 image to draw
* @param width Image width
* @param height Image height
* @param x Point horizontal coordinate
* @param y Point vertical coordinate  
* @param color A My_Draw_color to use
* @param space Size of the cross
* @param bold Width of the lines
*/
void my_draw_point(	unsigned char *im, 
			unsigned int width, 
			unsigned int height, 
			unsigned int x, 
			unsigned int y,
			My_draw_color color=MY_RED,
			int space=6,
			int bold=3
	);

void my_draw_rect(	unsigned char *im, 
			unsigned int width, 
			unsigned int height, 
			unsigned int tlx, 
			unsigned int tly, 
			unsigned int brx, 
			unsigned int bry, 
			My_draw_color color=MY_GREEN, 
			int bold=3
	);

#endif
